
 package Puz123;

################################################################
###                                                          ###
### SCaFoS                                                   ###
### Determination de la distance evolutive par Puzzle        ###
###                                                          ###
################################################################
### Version 4.40 du 2006/08/28                           BPH ###
################################################################


 use strict;
 use warnings;
 use Cwd;

 use Lib123;
 use Var123;
 use Gub123;
 
 use vars qw(@ISA @EXPORT);
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw/ SetPuzzleDist /;

 
################################################################


 my $bAff = '';       # texte pour affichage en mode graphique
 my $L;

 my %hSeqTmp;       # hachage des groupes pour creation eventuelle chimeres
                    #    ( nom groupe => [ 0:nb blancs min pour le groupe,
                    #                      1:hachage nom.seq=>seq,
                    #                      2:nb seq,
 my %hCmpFile;      # hachage de la composition du fichier
                    #    ( AA => Nb positions )
 my %hCmpSeq;       # hachage de la composition des sequences
                    #    ( nom sequence => { AA => Nb positions } )
 my @lAA = ( 'A','C','D','E','F','G','H','I','K','L','M','N','P','Q','R','S','T','V','W','Y' );
 my @lNuc = ( 'A','C','G','T' );
 
 
################################################################


### Calcul des distances par puzzle ou sequence la plus longue #
sub SetPuzzleDist {
    my ($ListEsp) = @_;
    my @T1;
    my $L;
    
    if ( $FlagChim == 1 ) {
        $L = $NameCef.".chim.log";
        if ( !open(LOG5,"> $L") ) { AffError("Can't open for writing file \'$L\' !"); }
        print LOG5 "MAKING OF CHIMERAS\n";
        }
        
    #** Maj fichier log pour composition des sequences ***
    if ( $FlagCmp == 1 ) {
        $L = $NameCef.".cmp";
        if ( !open(CMP,"> $L") ) { AffError("Can't open for writing file \'$L\' !"); }
        print CMP "-----------------------------------------------------------------------\n";
        print CMP " In directory : $FileRep\n";
        print CMP " Out directory : $CefRep\n";
        print CMP " File of species : $SpecFile\n";
        print CMP " ".scalar(localtime(time()))." \n";
        print CMP "-----------------------------------------------------------------------\n";
        }
        
    #** Utilisation des PUZZLE pour determiner meilleure sequence 
    #   (option precise pour determination de la sequence) ******
    if ( $FlagNoPuz == 0 ) {
        if ( $FlagIHM == 1 ) { ViewCurrent('Selection of the less divergent sequence within each OTU', 'darkblue', $Current); }
        #** construit fichier de param pour puzzle ***
        $L = $PuzRep."puzzle.par";
        if ( !open(PUZ,"> $L") )
            { AffError("Can't open for writing file \'$L\' !"); }
        if ( system("puzzle -version 2>puz.ver") == 0 ) {    # redirection de la sortie en erreur standard
            if ( open(VER, "puz.ver") ) {
                @T1 = <VER>;
                close VER;
                $L = join("", @T1) . ">";
                if ( $L =~ m/.*(\d)\.(\d).*/ ) {
                    if    ( int($1) < 5 ) { goto V51; }  # < 5.0
                    elsif ( int($1) > 5 ) { goto V52; }  # > 5.9
                    elsif ( int($2) < 2 ) { goto V51; }  # < 5.2
                    else                  { goto V52; }  # > 5.1
V51:
                    $L = "k\nk\n";                       # = 5.1
                    }
                else {
                    goto V52_PRESUM;
                    }
                }
            else {
V52_PRESUM:
                if ( $FlagIHM == 1 ) { ViewInfo("Warning", "Can't determinate version of PUZZLE, 5.2 is assumed \'$L\' !", 'red'); }
                else { print "   Can't determinate version of TREE-PUZZLE, 5.2 is assumed \'$L\' !"; }
V52:
                $L = "k\nk\nk\n";
                }
            if ( $FlagGamma == 1 ) { $L .= "w\nc\n4\n"; }
            $L .= "y\n";
            }
        if ( $^O eq 'MSWin32' ) { system("del /s /q puz.ver"); }
        else { system("rm -f puz.ver"); }
        print PUZ $L;
        close PUZ;
        
        #** Maj fichier log pour gestion des distances ***
        $L = $NameCef.".dist";
        if ( !open(LOG2,"> $L") ) { AffError("Can't open for writing file \'$L\' !"); }
        print LOG2 "-----------------------------------------------------------------------\n";
        print LOG2 " In directory : $FileRep\n";
        print LOG2 " Out directory : $CefRep\n";
        print LOG2 " File of species : $SpecFile\n";
        print LOG2 " ".scalar(localtime(time()))." \n";
        print LOG2 "-----------------------------------------------------------------------\n";
        
        #** construit le fichier .puz, appelle puzzle et traite les distances ***
        print LOG2 "\n\n MAXIMUM in-OTU DISTANCE FOR EACH OTU\n";
        print LOG2 "    AVERAGE OF out-OTU DISTANCE";
        if ( $FlagCmp == 1 )
            { print LOG2 " / VARIATION OF COMPOSITION FOR EACH SEQUENCE\n\n"; }
        else
            { print LOG2 " FOR EACH SEQUENCE\n"; }
        if ( $FlagVerb == 1 ) {    
            $bAff = "\n\n=======================================================================\n";
            $bAff .= "\n CALCULATE AVERAGE OF DISTANCES\n";
            }
        else
            { $bAff = "\n CALCULATE AVERAGE OF DISTANCES\n"; }
        if ( $FlagIHM == 1 ) { ViewDetails("\n$bAff", 0); }
        else { print "\n$bAff"; }
        for my $F ( sort keys %Fichiers ) {
            $bAff = "\n   === $F ==="; 
            if ( $FlagIHM == 1 ) { ViewDetails($bAff, 1); }
            elsif ( $FlagVerb == 1 ) { print $bAff; }
            print LOG2 "\n=== $F ===\n";
            if ( $FlagChim == 1 ) { print LOG5 "\n=== $F ===\n"; }
            if ( !open(ALI, $FileRep.$SepDir.$F) ) { AffError("Can't open for reading file \'$FileRep/$F\' !"); }
            WriteFilePuz($ListEsp, $F);
            close ALI;
            }
        print LOG2 "\n";
        close LOG2;
        }
    
    #** Utilisation de la sequence la plus longue
    #   (option rapide mais moins precise) *********************
    else {
        $bAff = "\n\n SELECTION OF LONGEST SEQUENCE\n"; 
        if ( $FlagIHM == 1 ) { 
            ViewCurrent('Selection of the longest sequence within each OTU', 'darkblue', $Current );
            ViewDetails($bAff, 0);
            }
        elsif ( $FlagVerb == 0 ) 
            { print "$bAff"; }
        for my $F ( sort keys %Fichiers ) {
            if ( $FlagVerb == 1 ) { 
                $bAff = "\n   === $F ==="; 
                if ( $FlagIHM == 1 ) { ViewDetails($bAff, 1); }
                else { print $bAff; }
                }

            if ( $FlagChim == 1 ) { print LOG5 "\n=== $F ===\n"; }
            if ( !open(ALI, $FileRep.$SepDir.$F) ) { AffError("Can't open for reading file \'$FileRep/$F\' !"); }
            WriteFilePuz($ListEsp, $F);
            close ALI;
            }
        }
        
    if ( $FlagCmp == 1 ) { close CMP; }
    if ( $FlagChim == 1 ) { close LOG5; }       
}


### Ecriture du fichier pour puzzle et sauvegarde de la concordance des especes ###
sub WriteFilePuz {
    my ($ListSpe, $FileAli) = @_;   # liste des especes retenues, nom du fichier Ali
    my $KeyFile = "";   # cle pour le hachage de correspondance %hCorres
    my $nbSeq = 0;      # nombre de sequences retenues par l utilisateur
    my $buf = "";       # sauvegarde des sequences pour .puz
    my $SeqName = "";   # nom de la sequence courante
    my $Id = 0;         # identifiant de la sequence pour .puz
    my $Pos = 0;        # position courante dans le fichier .ali
    my $L;
    my $SpecName = "";
    my $seq = "";
    my $GrpName = "";
    my $nbBlancs = 0 ;  # nb de ' ' ? - *
    my $bSpec;
    my %hTmp = ();
    my $Lg = $FicLen{$FileAli};
    my $MaxBlanc;
    my $flag;
    my $FullSize;
    my $NbSeqFile = 0;  # nb total de sequences dans le fichier;
    my $Name;
    
    %hSeqTmp = ();
    $FileAli =~ m/(.*)\.ali$/i;
    $KeyFile = $1;
    if ( $FlagChim == 1 )
        { $FullSize = sprintf("%.0f", $Lg * $MissPos / 100); }
    else
        { $FullSize = sprintf("%.0f", $Lg); }
    $MaxBlanc = sprintf("%.0f", $Lg * (1 - ($MinLen / 100)));

    while ( $L = <ALI> ) {
        # ligne de commentaire
        if ( $L =~ m/^#/ ) { next; }
        chomp($L);
        # ligne vide
        if ( $L eq "" ) { next; }
        
        #** recupere le nom de l'espece ************************
        if ( $L =~ m/^>(.*)/ ) {
            $L = $1;
            #-- stocke sequence precedente ---
            if ( $SeqName ne "" ) {
                $nbSeq = ConstHachSeq($SeqName, $seq, $Lg, $bSpec, $MaxBlanc, $FullSize, $nbSeq, $KeyFile);
                }
            #-- reinitialise variables ---
            $seq = "";
            $SeqName = "";
            
            # supprime commentaire en fin de ligne
            if ( $L =~ m/(.*)#(.+)/ ) { $L = $1; }
            $L =~ s/^\s*//;
            $L =~ s/\s*$//;
            $SpecName = SearchSpecName($L, "@");
            $bSpec = "<<<<<" . $SpecName . "<<<<<";   # elimine les especes dont le nom est entierement inclus dans le nom d'une autre espece
            # sauvegarde donnees pour espece retenue
            if ( $ListSpe =~ m/$bSpec/ ) { $SeqName = $L; }
            else { ($SeqName, $Id, $SeqName) = ("", 0, ""); }
            }
        
        #** recupere la sequence *******************************
        else {
            # espece non retenue : on passe a la suivante
            if ( $SeqName eq "" )
                { $Name = $SpecName; }
            # stockage de la sequence
            else { 
                $Name = $SeqName;
                $seq = $L; 
                }
            # determine composition de la sequence
            if ( $FlagCmp == 1 ) {
                $NbSeqFile++; 
                CmpSeq($Name, $L);
                }
            $SpecName = "";
            }
        }
        
    #** stocke derniere sequence *******************************
    if ( $SeqName ne "" ) {
        $nbSeq = ConstHachSeq($SeqName, $seq, $Lg, $bSpec, $MaxBlanc, $FullSize, $nbSeq, $KeyFile);
        }

    #** Ajoute composition au fichier de composition ***************
    if ( $FlagCmp == 1 ) { 
        my $x;
        my $delta;
        my $buf;
        my $f;
        my $nbPos;
        my $nb;
        
        #-- calcul composition moyenne du fichier ---
        $buf = "\n".pack("A30", "=== $KeyFile")." ";
        for $x ( keys %hCmpFile ) { $nbPos += $hCmpFile{$x}; }
        for $x ( sort keys %hCmpFile ) {
            $hCmpFile{$x} = (100 * $hCmpFile{$x} / $nbPos);
            $buf .= pack("A5", $x) . " ";
            }
        $buf .= "deviation from average";
        print CMP "$buf\n";
        #-- boucle sur les sequences retenues ---
        for $Name ( sort keys %{$hCorres{$KeyFile}} ) {
            $buf = pack("A30", $Name)." ";
            $delta = 0.0;
            $nb = 0.0;
            for $x ( keys %hCmpFile ) { $nb += $hCmpSeq{$Name}{$x}; }    # nb residus dans la sequence
            for $x ( sort keys %hCmpFile ) {
                $f = 100 * $hCmpSeq{$Name}{$x} / $nb;          # freq du residu dans la sequence
                $buf .= pack("A5", $f) . " ";
                $f -= $hCmpFile{$x};                           
                $delta += ($f * $f);
                }
            $buf .= pack("A6", $delta)."\n";
            print CMP $buf;
            $hCorres{$KeyFile}{$Name}[1] = $delta;
            }
        #-- ecriture de la moyenne par unite pour tout le fichier
        $buf = pack("A30", "Average")." ";
        for $x ( sort keys %hCmpFile )
            { $buf .=  pack("A5", $hCmpFile{$x}) . " "; }
        $buf .= "\n";
        print CMP $buf;
        }
                
    #** Construction hachages
    #     - pour sequences completes ou unique -> puzzle sinon la 1ere 
    #     - pour sequences partielles -> chimeres **************
    my $Sq;
    my $nb;
    if ( $FlagChim == 1 ) {
        foreach $GrpName ( sort keys %hSeqTmp ) {
            #** sequence unique ***
            if ( @{$hSeqTmp{$GrpName}}[2] == 1 ) {
                %hTmp = @{$hSeqTmp{$GrpName}[1]};
                my @T = keys %hTmp;
                $seq = $hTmp{$T[0]};
                ($flag, $nb) = TooShort($MaxBlanc, $seq);
                if ( $flag == 0 ) {
                    ++$nbSeq;
                    InitStock($KeyFile, $GrpName, 0);
                    StockToPuz($nbSeq, $seq, $KeyFile, $GrpName, $T[0]);
                    }
                next;
                }
            #** sequence complete ou quasi-complete -> puzzle ***
            if ( @{$hSeqTmp{$GrpName}}[0] <= $FullSize) {
                InitStock($KeyFile, $GrpName, 0);
                # stocke les sequences completes du groupe 
                %hTmp = @{$hSeqTmp{$GrpName}[1]};
                foreach $Sq ( keys %hTmp ) {
                    $seq = $hTmp{$Sq};
                    ($flag, $nb) = TooShort($FullSize, $seq);
                    if ( $flag == 0 ) {
                        # ajout identifiant et sequence dans le hachage
                        ++$nbSeq;
                        StockToPuz($nbSeq, $seq, $KeyFile, $GrpName, $Sq);
                        }
                    }
                }

            #** sequence partielle -> chimere ***
            else {
                %hTmp = @{$hSeqTmp{$GrpName}[1]};
                my $bOri;
                my $Chim = '?' x $Lg;
                my $nbBlancs;
                my $Buf;
                my $i = 0;
                my $l;
                my $Name = "";
                my @L;
                my $nbS = 0;
                my $bLog = '';
                for ( $i = 0; $i < @{$hSeqTmp{$GrpName}}[2]; $i++ ) {
                    $Buf = '';
                    $SeqName = '';
                    # recherche la sequence la plus longue non encore utilisee pour chimere
                    $nbBlancs = $Lg;
                    foreach $Sq ( keys %hTmp ) {
                        $seq = $hTmp{$Sq};
                        ($flag, $nb) = TooShort($nbBlancs, $seq);
                        if ( $flag == 0 ) {
                            $nbBlancs = $nb;
                            $Buf = $seq;
                            $SeqName = $Sq;
                            }
                        }
                    if ( length($SeqName) > 0 ) {
                        $hTmp{$SeqName} = '?' x $Lg;
                        $bOri = ' ' x $Lg;
                        # compare avec chimere en cours de construction
                        $nb = 0;
                        for ( $l = 0; $l < $Lg; $l++ ) {
                            if ( substr($Chim, $l, 1) eq '?' and substr($Buf, $l, 1) ne '?' ) {
                                # place le caractere dans la chimere
                                substr($Chim, $l, 1, substr($Buf, $l, 1));
                                # place le caractere dans la sequence d'origine
                                substr($bOri, $l, 1, substr($Buf, $l, 1));
                                # supprime le caractere des autres sequences
                                foreach $Sq ( keys %hTmp )
                                    { substr($hTmp{$Sq}, $l, 1, '?') ; }
                                $nb++;
                                }
                            }
                        # sequence entree dans chimere
                        if ( $nb > 0 ) {
                            $Name .= "$SeqName ($nb), ";
                            $bLog .= pack("A2A50", " ", $SeqName) . $bOri . "\n";
                            $nbS++;
                            }
                        }
                    }
                #-- 1 sequence : on la prend --
                if ( $nbS == 1 ) {
                    $hSeqChim{$KeyFile}{$GrpName} = $Chim;
                    $hNameChim{$KeyFile}{$GrpName} = substr($Name, 0, length($Name)-2);
                    }
                #-- plusieurs sequences : sauvegarde chimere ---
                else {
                    $hSeqChim{$KeyFile}{$GrpName} = $Chim;
                    $hNameChim{$KeyFile}{$GrpName} = "chimera <- " . $Name;
                    print LOG5 "\n$GrpName :\n$bLog" . pack("A2A50", " ", "chimera") . $Chim . "\n";
                    }
                }
            }
        }

    #** Appel de Puzzle ****************************************
    if ( $FlagNoPuz == 0 ) {
        # ecrit le fichier PUZ
        $buf = '';
        $nbSeq = 0;
        $Pos = 0;
        for ( sort keys %{$hSeqFile{$KeyFile}} ) {
            $buf .= @{$hSeqFile{$KeyFile}{$_}}[1];
            $nbSeq += @{$hSeqFile{$KeyFile}{$_}}[2];
            # correspondance entre Id sequence et ordre dans le fichier .puz
            for $Id ( @{$hSeqFile{$KeyFile}{$_}[5]} ) {
                $hIdent{$KeyFile}{$Id} = ++$Pos ;
                }
            }
        $L = $KeyFile.".puz";
        if ( LoadPuzzle($PuzRep, $L, $nbSeq, $FicLen{$FileAli}, $buf) == -1 ) { return(-1); }

        #** calcul de la moyenne des distances pour chaque 
        #   sequence ***********************************************
        CalculateAverage($KeyFile);
        my ($G, $S);
        # sauvegarde des resultats
        for $G ( sort keys %{$hSeqFile{$KeyFile}} ) {
            print LOG2 "\n ".$G." : max dist = ".pack("A6",@{$hSeqFile{$KeyFile}{$G}}[4])."\n";
            for $Id ( @{$hSeqFile{$KeyFile}{$G}[5]} ) {
                for $S ( sort keys %{$hCorres{$KeyFile}} ) {
                    if ( $Id == @{$hCorres{$KeyFile}{$S}}[0] ) {
                        print LOG2 "   ".$S." : distance = ".pack("A6",@{$hCorres{$KeyFile}{$S}}[2]);
                        if ( $FlagCmp == 1 ) { print LOG2 " / composition = ".pack("A6",@{$hCorres{$KeyFile}{$S}}[1])."\n"; }
                        else { print LOG2 "\n"; }
                        next;
                        }
                    }
                }
            }
        }
        
    return (0);
}


### Determine compostion de la sequence courante ###############
sub CmpSeq {
    my ($Name, $Sq) = @_;
    my ($x, $n, @L);

    if ( $TypeSeq eq 'protein' ) { @L = @lAA; }
    else                         { @L = @lNuc; }
    for $x ( @L ) {
        $n = 0;
        $n++ while ( $Sq =~ m/$x/gi );
        $hCmpFile{$x} += $n;
        $hCmpSeq{$Name}{$x} = $n;
        }
}


### Construit les hachages de sequences ########################
sub ConstHachSeq {
    my ($SeqName, $seq, $Lg, $bSpec, $MaxBlanc, $FullSize, $nbSeq, $KeyFile) = @_;
    my ($nbBlancs, $flag, $GrpName, $Id);
    my %hTmp = ();

    #-- cas des sequences trop courtes -------------------------
    $seq = pack("A".$Lg, $seq);
    # compte les 'non-caracteres'
    $seq =~ s/ |\*/\?/g;
    $nbBlancs = 0 ;
    ($flag, $nbBlancs) = TooShort($Lg, $seq);
    # si aucun caractere : on passe la sequence
    if ( $flag == 1 ) { return($nbSeq); }
        
    # recherche le groupe auquel appartient la sequence
    $GrpName = "";
    foreach ( keys %hGrp ) {
        if ( $hGrp{$_} =~ m/$bSpec/ ) {
            $GrpName = $_;
            last;
            }
        }
        
    #-- Possibilite de chimeres : on stocke les donnees --------
    if ( $FlagChim == 1 ) {
        # groupe pas encore stocke
        if ( !(exists($hSeqTmp{$GrpName})) ) {
            @{$hSeqTmp{$GrpName}}[0] = $nbBlancs;
            @{$hSeqTmp{$GrpName}[1]} = ();
            @{$hSeqTmp{$GrpName}}[2] = 0;
            }
        # stockage de la sequence dans le hachage temporaire
        if ( $nbBlancs < @{$hSeqTmp{$GrpName}}[0] )
            { @{$hSeqTmp{$GrpName}}[0] = $nbBlancs; 
            }
        %hTmp = @{$hSeqTmp{$GrpName}[1]};
        $hTmp{$SeqName} = $seq;
        @{$hSeqTmp{$GrpName}[1]} = %hTmp;
        @{$hSeqTmp{$GrpName}}[2] += 1;
        }
        
    #-- Recupere les sequences les + longues parmi celles 
    #   selectionnees par l 'utilisateur -----------------------
    else {
        # sequence trop courte : sequence suivante
        if ( $nbBlancs > $MaxBlanc ) {
            return($nbSeq); }
        # premier stockage pour ce groupe
        if ( !(exists($hSeqFile{$KeyFile}{$GrpName})) )
            { InitStock($KeyFile, $GrpName, $nbBlancs); }
        # ajout identifiant et sequence dans le buffer
        $Id = ++$nbSeq;
        StockToPuz($Id, $seq, $KeyFile, $GrpName, $SeqName);
        }
    return($nbSeq);
}


### Stockage sequence pour appel puzzle ########################
sub StockToPuz {
    my ($Id, $seq, $KeyFile, $GrpName, $Sq) = @_;                            
    my $buf = pack("A11", $Id).$seq."\n";
    if ( length($hSeqFile{$KeyFile}{$GrpName}[1]) > 0 )
        { $hSeqFile{$KeyFile}{$GrpName}[1] .= "\n"; }
    $hSeqFile{$KeyFile}{$GrpName}[1] .= $buf;       
    $hSeqFile{$KeyFile}{$GrpName}[2] += 1;
    $hSeqFile{$KeyFile}{$GrpName}[3] .= ($Sq."|");
    push @{$hSeqFile{$KeyFile}{$GrpName}[5]}, $Id;
    # initialise la correspondance entre Nom espece et Identifiant/Composition/Moyenne
    $hCorres{$KeyFile}{$Sq}[0] = $Id;
    $hCorres{$KeyFile}{$Sq}[1] = 0;
    $hCorres{$KeyFile}{$Sq}[2] = 0;
}


### Initialise un element dans le hachage hSeqFile #############
sub InitStock {
    my ($KeyFile, $GrpName, $nbBlancs) = @_;
    @{$hSeqFile{$KeyFile}{$GrpName}}[0] = $nbBlancs;
    @{$hSeqFile{$KeyFile}{$GrpName}}[1] = "";
    @{$hSeqFile{$KeyFile}{$GrpName}}[2] = 0;
    @{$hSeqFile{$KeyFile}{$GrpName}}[3] = "";
    @{$hSeqFile{$KeyFile}{$GrpName}}[4] = 0;
    @{$hSeqFile{$KeyFile}{$GrpName}[5]} = ();
    return(0);
}


### Calcule la distance moyenne pour chaque espece #############
sub CalculateAverage {
    my ($KeyFile) = @_;   # cle = nom du fichier  .ali/.puz.dist
    my $L = $PuzRep.$KeyFile.".puz.dist";
    if ( !open(DIST, $L) ) { AffError("Can't open for reading file \'$L\' !"); }
        
    # determine le nombre d especes dans le fichier .dist
    my $nbSpeDist = 0;
    $L = <DIST>;
    $L =~ s/\s+//;
    $nbSpeDist = EndLine($L);

    # calcule la distance moyenne pour chaque espece
    my $Id = "";          # Identifiant puzzle de l'espece courante
    my $SeqName = "";     # nom de la sequence courante
    my $GrpName = "";     # nom du groupe de la sequence courante
    my $PosList = "";     # Liste des positions des especes du groupe dans .puz
    my $nbLu = 0;         # nombre de distances lues
    my $nbPris = 0;       # nombre d'especes retenues pour calculer la moyenne
    my $sum = 0;          # somme des distances pour l'espece courante
    my @T = ();
    my $n;

    my $NoSeq = 0;        # numero de la sequence courante
    while ( $L = <DIST> ) {
        # gestion des fins de lignes dos/unix
        $L = EndLine($L);
        # decode la ligne
        $L =~ s/\s+/ /;
        @T = split(" ", $L);
        if ( $L =~ m/^\d/ ) {
            ++$NoSeq;
            $Id = $T[0];    # sauvegarde l'identifiant
            shift @T;       # supprime l'identifiant du tableau
            ($SeqName, $GrpName, $PosList) = SearchGrpSpe($Id, $KeyFile, 0);
            }
        my $Val = 0;
        
        for $Val ( @T ) {
            $nbLu++;
            # determine la distance max pour les sequences du groupe
            $b = "\|$nbLu\|";
            if ( index($PosList, $b) > -1 ) {
                if ( (@{$hSeqFile{$KeyFile}{$GrpName}}[2]) > 1 ) {
                    if ( @{$hSeqFile{$KeyFile}{$GrpName}}[4] < $Val ) {
                        @{$hSeqFile{$KeyFile}{$GrpName}}[4] = $Val;
                        }
                    }
                }
            # sinon additionne les distances des sequences hors groupe
            else {
                $sum += $Val;
                $nbPris++;
                }
            }
        # on a lu toutes les distances : on calcule la moyenne et on change de sequence
        if ( $nbLu == $nbSpeDist ) {
            $n = $sum / $nbPris;
            $hCorres{$KeyFile}{$SeqName}[2] = $n;
            ($Id, $nbLu, $nbPris, $sum, $SeqName, $PosList) = ("", 0, 0, 0, "", "");
            }
        }
    close DIST;
}


### Appel puzzle ###############################################
sub LoadPuzzle {
    my ($Rep, $File, $nbSeq, $lgSeq, $buf) = @_;

    # verifie si l'utilisateur a conserve au moins 4 especes
    if ( $nbSeq < 4 ) {
        if ( $nbSeq == 0 )
            { $bAff = "NO specified SPECIES in file \'".$Rep.$File."\' !\n\n"; }
        else
            { $bAff = "At least 4 sequences are necessary in file \'".$Rep.$File."\' to run PUZZLE!\n"; }
        if ( $FlagIHM == 1 ) { ViewInfo('Warning', $bAff, 'red'); }
        else { print "\n       $bAff"; }
        return (-1);
        }
    elsif ( $nbSeq > 256 ) {
        $bAff = "To many sequences in file \'".$Rep.$File."\' : $nbSeq sequences\nPUZZLE limites sequences number to 256\n";
        if ( $FlagIHM == 1 ) { ViewInfo('Warning', $bAff, 'red'); }
        else { print "\n       $bAff"; }
        return (-1);
        }
    # ecriture du fichier .puz
    my $InitRep = getcwd();
    chdir $Rep || die "       Can't change directory to \'$Rep\'\n";
    $L = $File;
    if ( !open(PUZ,"> $L") ) {
        if ( $FlagIHM == 1 ) { ViewInfo("Warning", "Can't open for writing file \'$L\' !", 'red'); }
        else { print "\n\n   Can't open for writing file \'$L\' !\n\n"; }
        return (-1);
        }
    print PUZ "$nbSeq $lgSeq\n";
    print PUZ "$buf\n";
    close PUZ;
    # appel de puzzle
    my $n;
    if ( $^O eq 'MSWin32' ) {
        $n = system("puzzle $File < puzzle.par > output.puz");
        system("del /f /q output.puz");
        }
    else
        { $n = system("puzzle $File < puzzle.par >/dev/null"); }
    if ( $n != 0 ) {
        $bAff = "\n       Can't load 'puzzle' for file \'".$Rep.$File."\' !\n\n";
        if ( $FlagIHM == 1 ) { ViewInfo('Warning', $bAff, 'red'); }
        else { print "\n       $bAff\n\n"; }
        chdir "../..";
        return (-1);
        }
    chdir $InitRep;

    return(0);
}


1;

### fin du fichier                                           ###
################################################################
